#include "aneu_meshloader.h"
#include <iostream>

inline void option_2(AneuMeshLoader& mesh)
{
    std::cout << "Enter node IDs: ";
    int ids[2];
    std::cin >> ids[0] >> ids[1];
    auto result = mesh.findElem(ids[0], ids[1]);
    std::cout << std::endl;
    for (auto& a : result)
    {
        std::cout << "Node ID: " << a.m_ID << "Material ID: " << a.m_mat_ID << "Nodes' IDs: ";
        for (auto& b : a.m_nodes) std::cout << ' ' << b;
        std::cout << std::endl;
    }
}

inline void option_3(AneuMeshLoader& mesh)
{
    std::cout << "Enter node IDs: ";
    int ids[3];
    std::cin >> ids[0] >> ids[1] >> ids[2];
    std::cout << std::endl;
    auto result = mesh.findElem(ids[0], ids[1], ids[2]);
    for (auto& a : result)
    {
        std::cout << "Node ID: " << a.m_ID << "Material ID: " << a.m_mat_ID << "Nodes' IDs: ";
        for (auto& b : a.m_nodes) std::cout << ' ' << b;
        std::cout << std::endl;
    }
}

inline void option_4(AneuMeshLoader& mesh)
{
    std::cout << "Enter surface ID: ";
    int id;
    std::cin >> id;
    std::cout << std::endl;
    std::vector<int> result;
    mesh.getNodesBySurfaceID(id, result);
    std::cout << "Nodes: " << std::endl;
    for (auto& a : result) std::cout << a << std::endl;
}

inline void option_5(AneuMeshLoader& mesh)
{
    std::cout << "Enter material ID: ";
    int id;
    std::cin >> id;
    std::cout << std::endl;
    std::vector<Element> result;
    mesh.getElementsByMatID(id, result);
    std::cout << "Nodes: " << std::endl;
    for (auto& a : result)
    {
        std::cout << "Node ID: " << a.m_ID << "Material ID: " << a.m_mat_ID << "Nodes' IDs: ";
        for (auto& b : a.m_nodes) std::cout << ' ' << b;
        std::cout << std::endl;
    }

}

inline void option_6(AneuMeshLoader& mesh)
{
    std::cout << "Enter surface ID: ";
    int id;
    std::cin >> id;
    std::cout << std::endl;
    std::vector<Surface> result;
    mesh.getSurfacesByID(id, result);
    std::cout << "Surfaces: " << std::endl;
    for (auto& a : result)
    {
        std::cout << "ID: " << a.m_ID << "Nodes' IDs: ";
        for (auto& b : a.m_nodes) std::cout << ' ' << b;
        std::cout << std::endl;
    }
}

inline void option_7(AneuMeshLoader& mesh)
{
    mesh.quadrateMeshElements();
    std::cout << "Mesh succsessfully quadrated" << std::endl;
}

inline void option_8(AneuMeshLoader& mesh)
{
    std::vector<std::vector<int>> result;
    mesh.getNeighbors(result);
    for (int i = 0; i != result.size(); i++)
    {
        std::cout << "Neighbors of node " << i << ":\t";
        for (int j = 0; j != result[i].size(); j++)
            std::cout << ' ' << result[i][j];
        std::cout << std::endl;
    }
}

int main(int argc, char** argv)
{
    char* p;
    AneuMeshLoader mesh;
    bool loaded = false;

    while (!loaded) {
        try {
            std::cout << "Import file: ";
            if (argc > 1)   std::cout << (p = argv[1]) << std::endl;
            else            std::cin >> (p = new char[100]);

            mesh.loadMesh(p);
            loaded = true;
        }
        catch (std::exception ex)
        {
            std::cout << ex.what() << std::endl;
        }
    }

    std::cout << "File successfully loaded" << std::endl << std::endl;

    int option = 0;
    bool flag = true;
    while (flag)
    {
        std::cout << "[1] Show loaded data" << std::endl;
        std::cout << "[2] Find element by 2 node IDs" << std::endl;
        std::cout << "[3] Find element by 3 node IDs" << std::endl;
        std::cout << "[4] Get nodes by surface ID" << std::endl;
        std::cout << "[5] Get elements by material ID" << std::endl;
        std::cout << "[6] Get surfaces by ID" << std::endl;
        std::cout << "[7] Quadrate mesh" << std::endl;
        std::cout << "[8] Get nodes neighbors" << std::endl;
        std::cout << "[9] Quit" << std::endl;
        std::cout << std::endl << "Enter number: ";
        std::cin >> option;
        std::cout << std::endl;

        switch (option)
        {
        case 1: std::cout << mesh;  break;
        case 2: option_2(mesh);     break;
        case 3: option_3(mesh);     break;
        case 4: option_4(mesh);     break;
        case 5: option_5(mesh);     break;
        case 6: option_6(mesh);     break;
        case 7: option_7(mesh);     break;
        case 8: option_8(mesh);     break;
        case 9: flag = false;       break;
        default:
            std::cout << "Incorrect input, try again" << std::endl;
        }

        std::cout << std::endl;
        system("pause");
        system("cls");
    };

    return 0;
}
